------------------------------------------------------
------------------------
--                         Copyright (C) 2006-200?
--                     Universidad de Cantabria, SPAIN
--                        http://www.ctr.unican.es/

--  RTCorba example: Sum
--  Node: Local
--  Protocol: RATO-RTEP
--  OS: Linux Fedora Core 4
--  Priority: Set at configuration time
--  Policies: RATO and Thread_Per_Target
--  Scheduling: Priorities

with MaRTE_OS;

--  Remote Object
with Suma;

--  Corba
with CORBA;
with CORBA.ORB;
with CORBA.Object;
with CORBA.Impl;

--  RTCorba
with RTCORBA;
with RTCORBA.Current;
with RTCORBA.Current.Helper;
with RTCORBA.PriorityMapping.Linear;

with PolyORB.RTCORBA_P.Setup;
--  End of RTCorba

--  System Configuration
with Client_Configuration;
pragma Elaborate_All (Client_Configuration);
with Configuration_File;

--  Debug and exceptions handling
with Ada.Exceptions;
with Kernel_Console; use Kernel_Console;

--  RTEP
with RTEP_IOR;
with PolyORB.RTEP_MAC;
with PolyORB.API.RTEP_MAC.Scheduling.Priorities;
with PolyORB.API.RTEP_MAC;    use PolyORB.API.RTEP_MAC;

with RTEP;                    use RTEP; --  Priority
with PolyORB.Tasking.Threads; use PolyORB.Tasking.Threads;

procedure Client is

   --  Short names
   package Sched renames PolyORB.API.RTEP_MAC.Scheduling.Priorities;
   package RTEP_MAC renames PolyORB.RTEP_MAC;

   --  Main task priority (check value in MaRTE OS)
   pragma Priority (8);

   --  Variables
   Break_Time         : Boolean := True; --  Time between requests
   Number_Of_Measures : Natural := 1000;--  Number of measures in each loop
   CLIENT_DEBUG       : Boolean := False; --  Console output

   --  Command line args
   Argv  : CORBA.ORB.Arg_List := CORBA.ORB.Command_Line_Arguments;
   idorb : CORBA.ORB.ORBid;

   --  Tasks to make requests: Each worker configures its parameters and make
   --  request to remote object SUM. The operation is Id + 2;
   task type Worker (Id : Natural; Principal : Boolean) is
      pragma Priority (30 + Id);
      entry Empieza;
      entry Prioridad;
   end Worker;

   task body Worker is
      --  Local variables
      Res     : CORBA.Long; --  Result of the operation
      Num1    : CORBA.Long; --  Operator
      Num2    : CORBA.Long := 2; --  Operator
      Ior_Ref : Suma.Ref; --  IOR to locate remote object

   begin

      accept Prioridad;

      --  Recover IOR from file (FAT not available in MaRTE)
      CORBA.ORB.String_To_Object
        (CORBA.To_CORBA_String (RTEP_IOR.IOR_Suma),
         Ior_Ref);

      --  Operator based on task identification
      Num1 := CORBA.Long (Id);

      --  ******************** RTEP ***************************

      declare

         P            : Sched.Priorities_Message_Scheduling_Parameters;
         P_Ref        : Sched.Priorities_Message_Scheduling_Parameters_Ref;
         Endpoint_Out : RTEP_MAC.Endpoint_Id;

      begin

         --  Scheduling API
         P.P_in := RTEP.Priority (30 + Id);
         Sched.Set_Scheduling_Params (P, P_Ref);

         --  Set up endpoint to send requests
         Sched.Create_Send_Endpoint
           (Params    => P_Ref,
            Event_ID  => Id,
            Dest_Node => 1,
            Channel   => RTEP_MAC.Channel (Id),
            Endpoint  => Endpoint_Out);

      end;

      --  ******************** RTEP ***************************

      accept Empieza;

      loop  --  Infinite looping

         for I in 1 .. Number_Of_Measures loop

            --  Remote request
            Res := Suma.SumaNum (Ior_Ref, Num1, Num2);

            if Break_Time then
               delay (0.01);
            end if;

         end loop;

         Kernel_Console.Put
           ("Worker: " &
            Natural'Image (Id) &
            " Result:" &
            CORBA.Long'Image (Res) &
            " Task:" &
            Image (Current_Task));

      end loop;

   end Worker;

   w1 : Worker (2, False);
   w2 : Worker (3, False);
   w3 : Worker (4, False);
   w4 : Worker (5, False);
   w5 : Worker (6, True);

begin

   --  Debug facilities
   PolyORB.API.RTEP_MAC.Enable_Debug (False);

   Kernel_Console.Put ("PolyORB over MaRTE (not debug)... Starting");
   idorb := CORBA.ORB.To_CORBA_String ("ORB");

   --  Init ORB
   Kernel_Console.Put ("Init ... ");
   CORBA.ORB.Init (idorb, Argv);

   Kernel_Console.Put ("Setting up RT enviroment ... ");

   w1.Prioridad;
   delay (0.1);
   w2.Prioridad;
   delay (0.1);
   w3.Prioridad;
   delay (0.1);
   w4.Prioridad;
   delay (0.1);
   w5.Prioridad;
   delay (0.1);

   w5.Empieza;
   w4.Empieza;
   w3.Empieza;
   w2.Empieza;
   w1.Empieza;

exception
   when E : others =>
      Kernel_Console.Put ("Received exception: ");
      Kernel_Console.Put (Ada.Exceptions.Exception_Information (E));

end Client;
