------------------------------------------------------
-----------
---------------
--                         Copyright (C) 2006-200?
--                     Universidad de Cantabria, SPAIN
--                        http://www.ctr.unican.es/

--  RTCorba example: Sum
--  Node: Remote
--  Protocol: IIOP
--  OS: Linux
--  RTCorba Priority: Client Propagated
--  Policies: Leader & Followers and Thread Pool
--  Scheduling: Priorities

with MaRTE_OS;

--  CORBA
with CORBA.Impl;
with CORBA.Object;
with CORBA.ORB;

with CORBA.Policy;
with CORBA;
with CORBA.Object.Policies;

with PortableServer.POA;
with RTPortableServer.POA;

with Suma.Impl; use Suma.Impl;

--  System Configuration
with Server_Configuration;
pragma Elaborate_All (Server_Configuration);

with Inicio;

--  Debug and exceptions handling
with Ada.Exceptions;
with Kernel_Console; use Kernel_Console;

--  RTEP
with PolyORB.API.RTEP_MAC;
with RTEP; use RTEP;

--  RTEP QoS
with PolyORB.QoS.RTEP_Parameters; use PolyORB.QoS.RTEP_Parameters;
pragma Elaborate_All (PolyORB.QoS.RTEP_Parameters);

--  RTEP Tagged Component
with PolyORB.GIOP_P.Tagged_Components.RTEP_Mac;
pragma Elaborate_All (PolyORB.GIOP_P.Tagged_Components.RTEP_Mac);

procedure Server is

   --  Variables
   IOR_ref     : CORBA.Object.Ref;
   RT_POA      : RTPortableServer.POA.Local_Ref;
   Object      : CORBA.Impl.Object_Ptr;
   Policy_List : CORBA.Policy.PolicyList;
   ResultP     : CORBA.Boolean;

   --  Command line arguments
   Argv  : CORBA.ORB.Arg_List := CORBA.ORB.Command_Line_Arguments;
   idorb : CORBA.ORB.ORBid;

begin

   --  Debug facilities
   PolyORB.API.RTEP_MAC.Enable_Debug (False);

   Put ("Running PolyORB over MaRTE OS...");
   New_Line;

   --  ORB Init
   idorb := CORBA.ORB.To_CORBA_String ("ORB");
   CORBA.ORB.Init (idorb, Argv);

   Put ("Init ... ");

   --  Create remote object
   Object := new Suma.Impl.Object;

   Put ("RTPOA ... ");

   --  RTPOA init
   Inicio.Servidor_RTCorba
     (Object => Object,
      RT_POA => RT_POA,
      IOR    => IOR_ref);

   Put ("Ok");
   New_Line;

   --  We need to rewrite the IOR because AP is not created at init phase
   IOR_ref :=
      PortableServer.POA.Servant_To_Reference
        (PortableServer.POA.Local_Ref (RT_POA),
         PortableServer.Servant (Object));

   Put (CORBA.To_Standard_String (CORBA.Object.Object_To_String (IOR_ref)));

   --  Launch the server
   Put ("Launching ... ");
   CORBA.ORB.Run;
   Put (" End of the ORB world ");

exception
   when E : others =>
      Put (" Received exception: ");
      Put (Ada.Exceptions.Exception_Information (E));

end Server;
